@@ -1,6 +1,6 @@
 /* ios.c - IO spaces for poke.  */
 
-/* Copyright (C) 2019, 2020, 2021, 2022, 2023 Jose E. Marchesi */
+/* Copyright (C) 2019, 2020, 2021, 2022, 2023, 2024 Jose E. Marchesi */
 
 /* This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -70,22 +70,24 @@
 struct ios
 {
   int id;
+  int zombie_p;
+  int num_sub_devs;
   char *handler;
   void *dev;
-  struct ios_dev_if *dev_if;
+  const struct ios_dev_if *dev_if;
   ios_off bias;
 
   struct ios *next;
 };
 
-/* Next available IOS id.  */
-
-static int ios_next_id = 0;
-
-/* List of IO spaces, and pointer to the current one.  */
-
-static struct ios *io_list;
-static struct ios *cur_io;
+struct ios_context
+{
+  int next_id;         /* Next available IOS id.  */
+  struct ios *io_list; /* List of all IO spaces.  */
+  struct ios *cur_io;  /* Pointer to the current IOS.  */
+  struct ios_dev_if *foreign_dev_if;
+  void *foreign_dev_if_data; /* User-defined data for foreign device.  */
+};
 
 /* The available backends are implemented in their own files, and
    provide the following interfaces.  */
@@ -101,44 +103,92 @@ extern struct ios_dev_if ios_dev_nbd; /* ios-dev-nbd.c */
 extern struct ios_dev_if ios_dev_proc; /* ios-dev-proc.c */
 #endif
 extern struct ios_dev_if ios_dev_sub; /* ios-dev-sub.c */
+#ifdef HAVE_MMAP
+extern struct ios_dev_if ios_dev_mmap; /* ios-dev-mmap.c */
+#endif
 
-static struct ios_dev_if *ios_dev_ifs[] =
+enum
+{
+  IOS_DEV_ZERO,
+  IOS_DEV_MEM,
+  IOS_DEV_STREAM,
+  IOS_DEV_NBD,
+  IOS_DEV_PROC,
+  IOS_DEV_SUB,
+  IOS_DEV_MMAP,
+  IOS_DEV_FILE, /* File must be last */
+};
+
+static const struct ios_dev_if *ios_dev_ifs[] =
   {
-   NULL, /* Optional foreign IOD.  */
-   &ios_dev_zero,
-   &ios_dev_mem,
-   &ios_dev_stream,
+    [IOS_DEV_ZERO] = &ios_dev_zero,
+    [IOS_DEV_MEM] = &ios_dev_mem,
+    [IOS_DEV_STREAM] = &ios_dev_stream,
 #ifdef HAVE_LIBNBD
-   &ios_dev_nbd,
+    [IOS_DEV_NBD] = &ios_dev_nbd,
+#else
+    [IOS_DEV_NBD] = NULL,
 #endif
 #ifdef HAVE_PROC
-   &ios_dev_proc,
+    [IOS_DEV_PROC] = &ios_dev_proc,
+#else
+    [IOS_DEV_PROC] = NULL,
 #endif
-   &ios_dev_sub,
-   /* File must be last */
-   &ios_dev_file,
-   NULL,
+    [IOS_DEV_SUB] = &ios_dev_sub,
+#ifdef HAVE_MMAP
+    [IOS_DEV_MMAP] = &ios_dev_mmap,
+#else
+    [IOS_DEV_MMAP] = NULL,
+#endif
+    [IOS_DEV_FILE] = &ios_dev_file, /* File must be last */
   };
 
-void
+static const size_t ios_dev_ifs_count
+    = sizeof (ios_dev_ifs) / sizeof (ios_dev_ifs[0]);
+
+static void *
+ios_context_data (ios_context ios_ctx, const struct ios_dev_if *dev_if)
+{
+  if (dev_if == ios_ctx->foreign_dev_if)
+    return ios_ctx->foreign_dev_if_data;
+  if (dev_if == ios_dev_ifs[IOS_DEV_SUB])
+    return ios_ctx;
+  return NULL;
+}
+
+ios_context
 ios_init (void)
 {
+  ios_context ios_ctx = calloc (1, sizeof (struct ios_context));
+
+  if (!ios_ctx)
+    return NULL;
   /* Nothing to do here... yet.  */
+  return ios_ctx;
 }
 
 void
-ios_shutdown (void)
+ios_shutdown (ios_context ios_ctx)
 {
+  ios inext;
+
+  if (!ios_ctx)
+    return;
   /* Close and free all open IO spaces.  */
-  while (io_list)
-    ios_close (io_list);
+  for (ios i = ios_ctx->io_list; i; i = inext)
+    {
+      inext = i->next;
+      ios_close (ios_ctx, i);
+    }
+  free (ios_ctx);
 }
 
 int
-ios_open (const char *handler, uint64_t flags, int set_cur)
+ios_open (ios_context ios_ctx, const char *handler, uint64_t flags,
+          int set_cur_p)
 {
   struct ios *io;
-  struct ios_dev_if **dev_if = NULL;
+  const struct ios_dev_if *dev_if = NULL;
   int iod_error = IOD_OK, error = IOS_ERROR;
 
   /* Allocate and initialize the new IO space.  */
@@ -146,39 +196,42 @@ ios_open (const char *handler, uint64_t flags, int set_cur)
   if (!io)
     return IOS_ENOMEM;
 
+  io->zombie_p = 0;
+  io->num_sub_devs = 0;
   io->handler = NULL;
   io->next = NULL;
   io->bias = 0;
 
   /* Look for a device interface suitable to operate on the given
      handler.  */
-  dev_if = ios_dev_ifs;
-  do
+  dev_if = ios_ctx->foreign_dev_if;
+  if (dev_if)
     {
-      if (*dev_if == NULL)
-        {
-          /* Skip the foreign IO device if it is not set.  */
-          ++dev_if;
-          continue;
-        }
-
-      io->handler = (*dev_if)->handler_normalize (handler, flags, &iod_error);
+      io->handler = dev_if->handler_normalize (handler, flags, &iod_error);
       if (iod_error != IOD_OK)
         goto error;
       if (io->handler)
-        break;
-
-      ++dev_if;
+        goto found;
+    }
+  for (size_t i = 0; i < ios_dev_ifs_count; ++i)
+    {
+      dev_if = ios_dev_ifs[i];
+      if (!dev_if)
+        continue; /* Skip unavailable devices.  */
+      io->handler = dev_if->handler_normalize (handler, flags, &iod_error);
+      if (iod_error != IOD_OK)
+        goto error;
+      if (io->handler)
+        goto found;
     }
-  while (*dev_if);
 
-  if (*dev_if == NULL)
-    goto error;
+  goto error;
 
-  io->dev_if = *dev_if;
+found:
+  io->dev_if = dev_if;
 
   /* Do not re-open an already-open IO space.  */
-  for (ios i = io_list; i; i = i->next)
+  for (ios i = ios_ctx->io_list; i; i = i->next)
     if (STREQ (i->handler, io->handler))
       {
         error = IOS_EOPEN;
@@ -186,20 +239,21 @@ ios_open (const char *handler, uint64_t flags, int set_cur)
       }
 
   /* Open the device using the interface found above.  */
-  io->dev = io->dev_if->open (handler, flags, &iod_error, io->dev_if->data);
+  io->dev = io->dev_if->open (handler, flags, &iod_error,
+                              ios_context_data (ios_ctx, io->dev_if));
   if (iod_error || io->dev == NULL)
     goto error;
 
   /* Increment the id counter after all possible errors are avoided.  */
-  io->id = ios_next_id++;
+  io->id = ios_ctx->next_id++;
 
   /* Add the newly created space to the list, and update the current
      space.  */
-  io->next = io_list;
-  io_list = io;
+  io->next = ios_ctx->io_list;
+  ios_ctx->io_list = io;
 
-  if (!cur_io || set_cur == 1)
-    cur_io = io;
+  if (!ios_ctx->cur_io || set_cur_p == 1)
+    ios_ctx->cur_io = io;
 
   return io->id;
 
@@ -215,7 +269,7 @@ ios_open (const char *handler, uint64_t flags, int set_cur)
 }
 
 int
-ios_close (ios io)
+ios_close (ios_context ios_ctx, ios io)
 {
   struct ios *tmp;
   int ret;
@@ -227,21 +281,29 @@ ios_close (ios io)
   ret = io->dev_if->close (io->dev);
 
   /* Unlink the IOS from the list.  */
-  assert (io_list != NULL); /* The list contains at least this IO space.  */
-  if (io_list == io)
-    io_list = io_list->next;
+  /* The list contains at least this IO space.  */
+  assert (ios_ctx->io_list != NULL);
+  if (ios_ctx->io_list == io)
+    ios_ctx->io_list = ios_ctx->io_list->next;
   else
     {
-      for (tmp = io_list; tmp->next != io; tmp = tmp->next)
+      for (tmp = ios_ctx->io_list; tmp->next != io; tmp = tmp->next)
         ;
       tmp->next = io->next;
     }
 
   /* Set the new current IO.  */
-  if (io == cur_io)
-    cur_io = io_list;
+  if (io == ios_ctx->cur_io)
+    ios_ctx->cur_io = ios_ctx->io_list;
 
-  free (io);
+  /* Re-use the ID if this IOS was the most-recently opened IOS.  */
+  if (ios_ctx->next_id == io->id + 1)
+    --ios_ctx->next_id;
+
+  if (io->num_sub_devs == 0)
+    free (io);
+  else
+    io->zombie_p = 1;
 
   return IOD_ERROR_TO_IOS_ERROR (ret);
 }
@@ -259,35 +321,64 @@ ios_handler (ios io)
 }
 
 ios
-ios_cur (void)
+ios_cur (ios_context ios_ctx)
 {
-  return cur_io;
+  return ios_ctx->cur_io;
 }
 
 void
-ios_set_cur (ios io)
+ios_set_cur (ios_context ios_ctx, ios io)
 {
-  cur_io = io;
+  ios_ctx->cur_io = io;
+}
+
+int
+ios_zombie_p (ios io)
+{
+  assert (io);
+  return io->zombie_p;
 }
 
 ios
-ios_search (const char *handler)
+ios_search (ios_context ios_ctx, const char *handler, uint32_t flags)
 {
   ios io;
+  int exact_p = (flags & 1) == IOS_SEARCH_F_EXACT;
 
-  for (io = io_list; io; io = io->next)
-    if (STREQ (io->handler, handler))
-      break;
+  if (handler[0] == '\0')
+    return NULL;
+
+  if (exact_p)
+    {
+      for (io = ios_ctx->io_list; io; io = io->next)
+        if (STREQ (io->handler, handler))
+          break;
+    }
+  else
+    {
+      ios ios_matched = NULL;
+      size_t n_matched = 0;
+
+      for (io = ios_ctx->io_list; io; io = io->next)
+        if (strstr (io->handler, handler) != NULL)
+          {
+            if (++n_matched > 1)
+              return NULL;
+            ios_matched = io;
+          }
+
+      io = ios_matched;
+    }
 
   return io;
 }
 
 ios
-ios_search_by_id (int id)
+ios_search_by_id (ios_context ios_ctx, int id)
 {
   ios io;
 
-  for (io = io_list; io; io = io->next)
+  for (io = ios_ctx->io_list; io; io = io->next)
     if (io->id == id)
       break;
 
@@ -300,6 +391,12 @@ ios_get_id (ios io)
   return io->id;
 }
 
+const char *
+ios_get_handler (ios io)
+{
+  return io->handler;
+}
+
 const char *
 ios_get_dev_if_name (ios io)
 {
@@ -319,9 +416,9 @@ ios_set_bias (ios io, ios_off bias)
 }
 
 ios
-ios_begin (void)
+ios_begin (ios_context ios_ctx)
 {
-  return io_list;
+  return ios_ctx->io_list;
 }
 
 bool
@@ -337,12 +434,12 @@ ios_next (const ios io)
 }
 
 void
-ios_map (ios_map_fn cb, void *data)
+ios_map (ios_context ios_ctx, ios_map_fn cb, void *data)
 {
   ios io;
   ios io_next;
 
-  for (io = io_list; io; io = io_next)
+  for (io = ios_ctx->io_list; io; io = io_next)
     {
       /* Note that the handler may close IO.  */
       io_next = io->next;
@@ -1631,24 +1728,41 @@ ios_get_dev (ios ios)
   return ios->dev;
 }
 
-struct ios_dev_if *
+const struct ios_dev_if *
 ios_get_dev_if (ios ios)
 {
   return ios->dev_if;
 }
 
 struct ios_dev_if *
-ios_foreign_iod (void)
+ios_foreign_iod (ios_context ios_ctx)
 {
-  return ios_dev_ifs[0];
+  return ios_ctx->foreign_dev_if;
 }
 
 int
-ios_register_foreign_iod (struct ios_dev_if *iod_if)
+ios_register_foreign_iod (ios_context ios_ctx, struct ios_dev_if *iod_if,
+                          void *data)
 {
-  if (ios_dev_ifs[0] != NULL)
+  if (ios_ctx->foreign_dev_if)
     return IOS_ERROR;
 
-  ios_dev_ifs[0] = iod_if;
+  ios_ctx->foreign_dev_if = iod_if;
+  ios_ctx->foreign_dev_if_data = data;
   return IOS_OK;
 }
+
+void
+ios_inc_sub_dev (ios io)
+{
+  ++io->num_sub_devs;
+}
+
+void
+ios_dec_sub_dev (ios io)
+{
+  assert (io->num_sub_devs != 0);
+  --io->num_sub_devs;
+  if (io->zombie_p && io->num_sub_devs == 0)
+    free (io);
+}
