@@ -278,20 +278,21 @@ poked_autocmpl_send_one (pk_val pkind, pk_val pstring)
         err (1, "bufb_append() failed");                                      \
     }                                                                         \
   while (0)
-#define FUNC                                                                  \
-  (str_kind == PLET_AUTOCMPL_IDENT ? pk_completion_function                   \
-                                   : pk_ios_completion_function)
+#define COMPLETE(state)                                                 \
+  (str_kind == PLET_AUTOCMPL_IDENT                                      \
+   ? pk_completion_function (pkc, string, (state))                      \
+   : pk_ios_completion_function (pkc, "", string, (state)))
 
   APPEND (string);
-  if ((candidate = FUNC (pkc, string, 0)) != NULL)
+  if ((candidate = COMPLETE (0)) != NULL)
     {
       APPEND (candidate);
-      while ((candidate = FUNC (pkc, string, 1)) != NULL)
+      while ((candidate = COMPLETE (1)) != NULL)
         APPEND (candidate);
     }
   usock_out (srv, USOCK_CHAN_OUT_AUTOCMPL, kind, b.mem, b.cur - b.mem);
 
-#undef FUNC
+#undef COMPLETE
 #undef APPEND
 
   bufb_free (&b);
@@ -701,18 +702,18 @@ tifbuf_init(void)
 #endif
 
 static void
-tif_flush (void)
+tif_flush (pk_compiler pkc __attribute__ ((unused)))
 {
 }
 static void
-tif_puts (const char *s)
+tif_puts (pk_compiler pkc __attribute__ ((unused)), const char *s)
 {
   if (poked_options.debug_p)
     printf (">(p) '%s'\n", s);
   usock_out (srv, termout_chan, termout_cmdkind, s, strlen (s) + 1);
 }
 static void
-tif_printf (const char *fmt, ...)
+tif_printf (pk_compiler pkc __attribute__ ((unused)), const char *fmt, ...)
 {
   va_list ap;
   char *data = NULL;
@@ -730,7 +731,8 @@ tif_printf (const char *fmt, ...)
   free (data);
 }
 static void
-tif_indent (unsigned int level, unsigned int step)
+tif_indent (pk_compiler pkc __attribute__ ((unused)), unsigned int level,
+            unsigned int step)
 {
   size_t len = /*newline*/ 1u + step * level;
   char *data;
@@ -743,31 +745,32 @@ tif_indent (unsigned int level, unsigned int step)
   free (data);
 }
 static void
-tif_class (const char *name)
+tif_class (pk_compiler pkc __attribute__ ((unused)), const char *name)
 {
   if (termout_chan == USOCK_CHAN_OUT_OUT)
     usock_out (srv, termout_chan, OUTCMD_CLS_BEGIN, name, strlen (name) + 1);
 }
 static int
-tif_class_end (const char *name)
+tif_class_end (pk_compiler pkc __attribute__ ((unused)), const char *name)
 {
   if (termout_chan == USOCK_CHAN_OUT_OUT)
     usock_out (srv, termout_chan, OUTCMD_CLS_END, name, strlen (name) + 1);
   return 1;
 }
 static void
-tif_hlink (const char *name, const char *id)
+tif_hlink (pk_compiler pkc __attribute__ ((unused)), const char *name,
+           const char *id)
 {
   (void)name;
   (void)id;
 }
 static int
-tif_hlink_end (void)
+tif_hlink_end (pk_compiler pkc __attribute__ ((unused)))
 {
   return 1;
 }
 static struct pk_color
-tif_color (void)
+tif_color (pk_compiler pkc __attribute__ ((unused)))
 {
   static struct pk_color c = {
     .red = 0,
@@ -777,7 +780,7 @@ tif_color (void)
   return c;
 }
 static struct pk_color
-tif_bgcolor (void)
+tif_bgcolor (pk_compiler pkc __attribute__ ((unused)))
 {
   static struct pk_color c = {
     .red = 255,
@@ -787,12 +790,12 @@ tif_bgcolor (void)
   return c;
 }
 static void
-tif_color_set (struct pk_color c)
+tif_color_set (pk_compiler pkc __attribute__ ((unused)), struct pk_color c)
 {
   (void)c;
 }
 static void
-tif_bgcolor_set (struct pk_color c)
+tif_bgcolor_set (pk_compiler pkc __attribute__ ((unused)), struct pk_color c)
 {
   (void)c;
 }
@@ -966,7 +969,8 @@ poked_init (int pdap_version)
     char *newpath = pk_str_concat (pk_string_str (load_path), ":",
                                    poked_appdir, ":", poke_picklesdir, NULL);
 
-    pk_decl_set_val (pkc, "load_path", pk_make_string (newpath));
+    pk_decl_set_val (pkc, "load_path",
+                     pk_make_string (pkc, newpath));
     free (newpath);
   }
 
@@ -995,8 +999,9 @@ poked_init (int pdap_version)
     }
 
   pk_decl_set_val (pkc, "__poked_pdap_version",
-                   pk_make_int (pdap_version, 32));
-  pk_decl_set_val (pkc, "__poked_libpoke_version", pk_make_string (VERSION));
+                   pk_make_int (pkc, pdap_version, 32));
+  pk_decl_set_val (pkc, "__poked_libpoke_version",
+                   pk_make_string (pkc, VERSION));
 
   return OK;
 }
